@@ -1,3 +1,8 @@
+from . import common
+from . import dsp
+from . import sampling
+from . import stream
+
 import numpy as np
 import itertools
 import logging
@@ -5,10 +10,6 @@ import subprocess
 
 log = logging.getLogger(__name__)
 
-from . import common
-from . import dsp
-from . import sampling
-
 
 def volume_controller(cmd):
     def controller(level):
@@ -35,7 +36,7 @@ def send(config, dst, volume_cmd=None, gain=1.0, limit=None):
 
 def frame_iter(config, src, frame_length):
     frame_size = frame_length * config.Nsym * config.sample_size
-    omegas = 2 * np.pi * config.frequencies / config.Fs
+    omegas = 2 * np.pi * np.array(config.frequencies) / config.Fs
 
     while True:
         data = src.read(frame_size)
@@ -110,7 +111,7 @@ def iter_window(iterable, size):
             yield block
 
 
-def recv(config, src, verbose=False, volume_cmd=None):
+def recv(config, src, verbose=False, volume_cmd=None, dump_audio=None):
     fmt = '{0.freq:6.0f} Hz: {0.msg:20s}'
     if verbose:
         fields = ['total', 'rms', 'coherency', 'peak']
@@ -118,11 +119,12 @@ def recv(config, src, verbose=False, volume_cmd=None):
 
     volume_ctl = volume_controller(volume_cmd)
 
+    if dump_audio:
+        src = stream.Dumper(src, dump_audio)
     result_iterator = detector(config=config, src=src)
     result_iterator = volume_calibration(result_iterator, volume_ctl)
-    result_iterator = iter_window(result_iterator, size=3)
-    for r in result_iterator:
+    for _prev, curr, _next in iter_window(result_iterator, size=3):
         # don't log errors during frequency changes
-        if r[0].success and r[2].success and r[0].freq != r[2].freq:
-            r[1].msg = r[1].msg if r[1].success else 'frequency change'
-        log.info(fmt.format(r[1]))
+        if _prev.success and _next.success and _prev.freq != _next.freq:
+            curr.msg = curr.msg if curr.success else 'frequency change'
+        log.info(fmt.format(curr))
